aboutsummaryrefslogtreecommitdiffstats
path: root/gson/src/main/java
diff options
context:
space:
mode:
Diffstat (limited to 'gson/src/main/java')
-rw-r--r--gson/src/main/java/org/onap/policy/common/gson/DoubleConverter.java117
-rw-r--r--gson/src/main/java/org/onap/policy/common/gson/MapDoubleAdapterFactory.java90
2 files changed, 141 insertions, 66 deletions
diff --git a/gson/src/main/java/org/onap/policy/common/gson/DoubleConverter.java b/gson/src/main/java/org/onap/policy/common/gson/DoubleConverter.java
new file mode 100644
index 00000000..81803ff2
--- /dev/null
+++ b/gson/src/main/java/org/onap/policy/common/gson/DoubleConverter.java
@@ -0,0 +1,117 @@
+/*
+ * ============LICENSE_START=======================================================
+ * ONAP
+ * ================================================================================
+ * Copyright (C) 2019 AT&T Intellectual Property. All rights reserved.
+ * ================================================================================
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ * http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ * ============LICENSE_END=========================================================
+ */
+
+package org.onap.policy.common.gson;
+
+import java.util.ArrayList;
+import java.util.List;
+import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Set;
+
+/**
+ * Converter for Double values. By default, GSON treats all Objects that are numbers, as
+ * Double. This converts Doubles to Integer or Long, if possible. It converts stand-alone
+ * Doubles, as well as those found within Arrays and Maps.
+ */
+public class DoubleConverter {
+
+ private DoubleConverter() {
+ // do nothing
+ }
+
+ /**
+ * Performs in-place conversion of all values in a list.
+ *
+ * @param list the list whose values are to be converted
+ */
+ public static void convertFromDouble(List<Object> list) {
+ if (list == null) {
+ return;
+ }
+
+ List<Object> original = new ArrayList<>(list);
+
+ list.clear();
+ original.forEach(item -> list.add(convertFromDouble(item)));
+ }
+
+ /**
+ * Performs in-place conversion of all values in a map.
+ *
+ * @param map the map whose values are to be converted
+ */
+ public static void convertFromDouble(Map<String, Object> map) {
+ if (map == null) {
+ return;
+ }
+
+ Set<Entry<String, Object>> set = map.entrySet();
+
+ for (Entry<String, Object> entry : set) {
+ entry.setValue(convertFromDouble(entry.getValue()));
+ }
+ }
+
+ /**
+ * Converts a value. If the value is a List, then it recursively converts the
+ * entries of the List. Likewise with a map, however, the map is converted in place.
+ *
+ * @param value value to be converted
+ * @return the converted value
+ */
+ @SuppressWarnings({"unchecked"})
+ public static Object convertFromDouble(Object value) {
+ if (value == null) {
+ return value;
+ }
+
+ if (value instanceof List) {
+ convertFromDouble((List<Object>) value);
+ return value;
+ }
+
+ if (value instanceof Map) {
+ convertFromDouble((Map<String, Object>) value);
+ return value;
+ }
+
+ if (!(value instanceof Double)) {
+ return value;
+ }
+
+ Double num = (Double) value;
+ long longval = num.longValue();
+
+ if (Double.compare(num.doubleValue(), longval) != 0) {
+ // it isn't integral - return unchanged value
+ return value;
+ }
+
+ // it's integral - determine if it's an integer or a long
+ int intval = (int) longval;
+
+ if (intval == longval) {
+ return intval;
+ }
+
+ return longval;
+ }
+}
diff --git a/gson/src/main/java/org/onap/policy/common/gson/MapDoubleAdapterFactory.java b/gson/src/main/java/org/onap/policy/common/gson/MapDoubleAdapterFactory.java
index e56f9dd3..bd031999 100644
--- a/gson/src/main/java/org/onap/policy/common/gson/MapDoubleAdapterFactory.java
+++ b/gson/src/main/java/org/onap/policy/common/gson/MapDoubleAdapterFactory.java
@@ -29,33 +29,47 @@ import com.google.gson.stream.JsonWriter;
import java.io.IOException;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
+import java.util.List;
import java.util.Map;
-import java.util.Map.Entry;
-import java.util.Set;
/**
* Adapter factory for Map&lt;String,Object&gt;. By default, GSON treats all Objects, that
* are numbers, as Double. This walks the map and converts Doubles to Integer or Long, if
* possible.
*/
public class MapDoubleAdapterFactory implements TypeAdapterFactory {
@Override
public <T> TypeAdapter<T> create(Gson gson, TypeToken<T> type) {
- if (type.getRawType() != Map.class) {
+ if (!isMapType(type) && !isListType(type)) {
return null;
}
+ TypeAdapter<T> delegate = gson.getDelegateAdapter(this, type);
+
+ return new MapAdapter<>(delegate);
+ }
+
+ private <T> boolean isMapType(TypeToken<T> type) {
+ if (type.getRawType() != Map.class) {
+ return false;
+ }
+
Type[] actualParams = ((ParameterizedType) type.getType()).getActualTypeArguments();
// only supports Map<String,Object>
- if (actualParams[0] != String.class || actualParams[1] != Object.class) {
- return null;
+ return (actualParams[0] == String.class && actualParams[1] == Object.class);
+ }
+
+ private <T> boolean isListType(TypeToken<T> type) {
+ if (type.getRawType() != List.class) {
+ return false;
}
- TypeAdapter<T> delegate = gson.getDelegateAdapter(this, type);
+ Type[] actualParams = ((ParameterizedType) type.getType()).getActualTypeArguments();
- return new MapAdapter<>(delegate);
+ // only supports List<Object>
+ return (actualParams[0] == Object.class);
}
/**
@@ -88,65 +102,9 @@ public class MapDoubleAdapterFactory implements TypeAdapterFactory {
public T read(JsonReader in) throws IOException {
T value = delegate.read(in);
- @SuppressWarnings("rawtypes")
- Map map = (Map) value;
-
- convertFromDouble(map);
+ DoubleConverter.convertFromDouble(value);
return value;
}
-
- /**
- * Performs conversion of all values in a map.
- *
- * @param map the map whose values are to be converted
- */
- @SuppressWarnings("rawtypes")
- private void convertFromDouble(Map map) {
-
- @SuppressWarnings("unchecked")
- Set<Entry> set = map.entrySet();
-
- for (Entry entry : set) {
- convertFromDouble(entry);
- }
- }
-
- /**
- * Converts an entry's value. If the value is a Map, then it recursively converts
- * the entries of the map.
- *
- * @param entry entry whose value is to be converted
- */
- @SuppressWarnings({"unchecked", "rawtypes"})
- private void convertFromDouble(Entry entry) {
- Object obj = entry.getValue();
-
- if (obj instanceof Map) {
- convertFromDouble((Map) obj);
- return;
- }
-
- if (!(obj instanceof Double)) {
- return;
- }
-
- Double num = (Double) obj;
- long longval = num.longValue();
-
- if (Double.compare(num.doubleValue(), longval) == 0) {
- // it's integral - determine if it's an integer or a long
- int intval = (int) longval;
-
- if (intval == longval) {
- // it fits in an integer
- entry.setValue(intval);
-
- } else {
- // doesn't fit in an integer - must be a long
- entry.setValue(longval);
- }
- }
- }
}
}
@@ -29,33 +29,47 @@ import com.google.gson.stream.JsonWriter;
/**
- * Adapter factory for Map&lt;String,Object&gt;. By default, GSON treats all Objects, that
- * are numbers, as Double. This walks the map and converts Doubles to Integer or Long, if
+ * Adapter factory for Map&lt;String,Object&gt; and List&lt;String&gt;. By default, GSON treats all Objects, that
+ * are numbers, as Double. This recursively walks a map/list and converts Doubles to Integer or Long, if
* possible.
*/